import 'dart:async';

import 'package:circumstellar/src/models/event_dispatcher.dart';
import 'package:circumstellar/src/rest/discord_rest.dart';
import 'package:circumstellar/src/rest/request_options.dart';
import 'package:circumstellar/src/rest/rest_response.dart';
import 'package:circumstellar/src/ws/discord_ws.dart';
import 'package:w_transport/w_transport.dart';

import 'discord_constants.dart';

import 'models/auth_set.dart';

/// Represents a connection to the two Discord APIs.
class DiscordClient {
  /// The current authentication properties.
  DiscordAuthSet auth;

  /// The w_transport transport platform - either VM or Web.
  TransportPlatform transportPlatform;

  /// The current WebSocket wrapper.
  DiscordWebSocketClient ws;

  /// The current REST wrapper.
  RestClient rest;

  /// The current event dispatcher.
  EventDispatcher dispatcher;

  /// Default constructor.
  DiscordClient(this.auth, this.transportPlatform) {
    this.rest = new RestClient(this);
    this.ws = new DiscordWebSocketClient(this, rest);
    this.dispatcher = new EventDispatcher(this);
  }

  /// Starts the WebSocket and REST APIs and attempts to contact Discord.
  Future start() async {
    await this.ws.connect();
  }

  /// Stops all APIs.
  void stop() {
    this.ws.stop();
  }

  // Utility functions

  /// Creates a message in the specified channel (ID)
  Future<RestRequestResult> createMessage(String channelId, String contentArg,
      {bool tts = false,
      Map<String, dynamic> embed = null,
      RequestOptions options = null}) async {
    return await this.rest.executeRoute(
        ApiRoutes.channels.createMessage(channelId),
        {'content': contentArg, 'embed': embed});
  }
}
