import 'dart:async';

import 'package:circumstellar/circumstellar.dart';
import 'package:circumstellar/src/discord_client.dart';
import 'package:circumstellar/src/models/discord_objects/channels/channel_selector.dart';
import 'package:circumstellar/src/models/entity.dart';
import 'package:circumstellar/src/models/discord_objects/channels/guild_text_channel.dart';
import 'package:circumstellar/src/models/discord_objects/user.dart';

class Message extends Entity<String> {
  /// The channel that this message was sent in.
  GuildTextChannel channel = null;

  /// The author of this message.
  User author = null;

  /// Content of this message.
  String content = '';

  /// Whether this message was a TTS message (Text-To-Speech)
  bool isTTS = false;

  /// Whether this message mentioned everyone
  bool mentionsEveryone = false;

  /// List of users who have been mentioned
  List<User> mentions = [];

  /// Whether this message has been pinned
  bool pinned = false;

  /// Creates a new [Message] from a JSON map.
  static Future<Message> createFromData(
      Map<String, dynamic> data, DiscordClient client) async {
    var message = new Message.empty(client)
      ..id = data['id']
      ..content = data['content']
      ..author = await User.createFromData(data['author'], client)
      ..isTTS = data['tts']
      ..mentionsEveryone = data['mention_everyone']
      ..pinned = data['pinned'];
    var mentions0 = new List.from(data['mentions']);
    mentions0.forEach((mention1) async {
      message.mentions.add(await User.createFromData(mention1, client));
    });

    var channelResponse = await client.rest
        .executeRoute(ApiRoutes.channels.getChannelWithId(data['channel_id']));
    if (channelResponse.isSuccess) {
      message.channel =
          await GuildTextChannel.createFromData(channelResponse.json, client);
    }
    return message;
  }

  Message.empty(DiscordClient client) : super.empty(client);
}
