import 'dart:async';

import 'package:circumstellar/src/discord_client.dart';
import 'package:circumstellar/src/models/entity.dart';

class User extends Entity<String> {
  /// The user's username, not unique across Discord
  String username = '';

  /// The user's discriminator, their 4-digit Discord tag
  String discriminator = '';

  /// The integer value of the user's discriminator
  int discriminatorValue = 0;

  /// Whether the user belongs to an OAuth2 application
  bool isBot = false;

  /// Whether the user has Two-Factor Authentication (2FA) enabled
  bool twoFactorEnabled = false;

  User.empty(DiscordClient client) : super.empty(client);

  static Future<User> createFromData(
      Map<String, dynamic> json, DiscordClient client) async {
    var user = new User.empty(client);
    user.id = json['id'];
    user.username = json['username'];
    user.discriminator = json['discriminator'];
    user.discriminatorValue = int.parse(user.discriminator);
    user.isBot = json['bot'];
    user.twoFactorEnabled = json['mfa_enabled'];
    return user;
  }
}
