import 'package:circumstellar/circumstellar.dart';
import 'package:circumstellar/src/models/entity.dart';

class ObjectCache<TObjectId> {
  DiscordClient _client;
  Map<TObjectId, Entity<TObjectId>> _cache = {};

  ObjectCache(this._client);

  /// Adds [object] to the internal cache.
  ///
  /// Objects in the cache are mapped by their ID ([TObjectId]), so if another object
  /// exists with the same ID, it will be overwritten.
  void add(Entity<TObjectId> object) {
    _cache[object.id] = object;
  }

  /// Retrieves the object with ID [id] from the cache.
  ///
  /// Returns null if the object does not exist.
  Entity<TObjectId> get(TObjectId id) {
    return _cache[id];
  }

  /// Checks if the cache contains a certain object of ID [id].
  bool contains(TObjectId id) {
    return (_cache[id] != null);
  }
}
