import 'dart:async';
import 'dart:convert';

import 'package:circumstellar/src/rest/rest_response.dart';
import 'package:w_transport/w_transport.dart' as transport;
import '../discord_client.dart';
import '../circumstellar_constants.dart';
import '../discord_constants.dart';

/// A client to connect to Discord's REST endpoints.
class RestClient {
  /// The owner of this REST wrapper.
  DiscordClient _client;

  /// Default constructor.
  RestClient(this._client);

  /// Handles a request after it has been terminated due to a failure.
  Future handleRequestError(transport.RequestException err) async {
    print('Request errored: ' + err.message);
    print('Request failed trying to access ' +
        err.uri.toString() +
        ' with method ' +
        err.method);
    print(err.response.statusText);
  }

  /// Creates and executes a JSON request based on an API route.
  Future<RestRequestResult> executeRoute(ApiRoute route,
      [Map<String, dynamic> jsonParams]) async {
    var request =
        new transport.JsonRequest(transportPlatform: _client.transportPlatform)
          ..uri = Uri.parse(route.buildRouteUrl())
          ..headers = {
            'Authorization': buildAuth(),
            'User-Agent': buildUserAgent()
          };
    if (route.method == "POST" ||
        route.method == "PUT" ||
        route.method == "PATCH") {
      request.body = jsonParams;
    }

    try {
      var response = await request.send(route.method);
      return new RestRequestResult.success(response);
    } on transport.RequestException catch (requestErr) {
      await handleRequestError(requestErr);
      return new RestRequestResult.failure(requestErr);
    }
  }

  /// Builds an authentication string for API requests.
  String buildAuth() => '${_getTokenPrefix()} ${_client.auth.token}';

  /// Converts a TokenType to an adequate string for the API.
  String _getTokenPrefix() =>
      _client.auth.tokenType.toString().replaceAll('TokenType.', '');

  /// Builds a compatible User-Agent for Discord.
  String buildUserAgent() =>
      "DiscordBot (${CircumstellarInfo.url}, ${CircumstellarInfo.version})";
}
