import 'package:w_transport/w_transport.dart' as transport;

/// Represents a result from a REST request.
class RestRequestResult {
  /// Whether this request succeeded.
  bool isSuccess;

  /// If it succeeded, the returned data as a JSON map.
  Map<String, dynamic> json;

  /// If it succeeded, the returned data as a string.
  String dataAsString;

  /// The status text of the respnose.
  String statusText;

  /// The status code of the response.
  int statusCode;

  /// If it failed, the exception associated.
  transport.RequestException exception;

  /// If it failed, the error message.
  String errorMessage;

  /// If it failed and one exists, the original error.
  Object originalError;

  /// Create a result based off of a successful response.
  RestRequestResult.success(transport.Response response) {
    isSuccess = true;
    json = response.body.asJson();
    dataAsString = response.body.asString();
    statusCode = response.status;
    statusText = response.statusText;
  }

  /// Create a result based off of a failed request, that threw an exception.
  RestRequestResult.failure(transport.RequestException err) {
    isSuccess = false;
    exception = err;
    errorMessage = err.message;
    originalError = err.error;
  }
}
