import 'dart:async';

import 'package:circumstellar/circumstellar.dart';
import 'package:circumstellar/src/models/ws_payload.dart';

/// Automatically sends heartbeats on a predetermined interval.
class WSHeartbeater {
  /// The client that owns this heartbeater.
  DiscordClient _client;

  /// The predetermined interval to heartbeat (in milliseconds)
  int hbInterval;

  /// The Dart timer that handles starting the hearbeat.
  Timer _hbTimer;

  /// Default constructor
  WSHeartbeater(this._client, this.hbInterval) {}

  /// Begins heartbeating on the interval specified.
  void begin() {
    _hbTimer = new Timer.periodic(
        new Duration(milliseconds: this.hbInterval), doHeartbeat);
  }

  /// Executed when the timer signals that the heartbeat interval has passed.
  Future doHeartbeat(Timer timer) async {
    var payload = new WSPayloadData();
    payload.opCode = DiscordGatewayStatic.opCodes[1];
    payload.innerPayload = _client.ws.currentSeq;
    _client.ws.sendMessage(payload);
  }

  /// Stops heartbeating.
  void stop() {
    _hbTimer.cancel();
    _hbTimer = null;
  }
}
