import '../discord_constants.dart';

/// Represents a bi-directional WebSocket data object that can be sent or received.
class WSPayloadData {
  bool innerPayloadIsJsonObject = false;
  OPCodeDescription opCode;
  dynamic innerPayload;
  int sequenceNumber;
  String eventName;

  /// Constructs a WebSocket payload data object out of a Map received from Discord.
  static WSPayloadData build(Map<String, dynamic> jsonObject) {
    var payload = new WSPayloadData();
    if (jsonObject.containsKey('t')) {
      payload.eventName = jsonObject['t'];
    }
    if (jsonObject.containsKey('s')) {
      payload.sequenceNumber = jsonObject['s'];
    }
    if (jsonObject['d'].runtimeType ==
        (new Map<String, dynamic>()).runtimeType) {
      payload.innerPayloadIsJsonObject = true;
    }
    payload.innerPayload = jsonObject['d'];
    payload.opCode = DiscordGatewayStatic.opCodes[jsonObject['op']];
    return payload;
  }

  /// Constructs this object into a Map, so it can be sent to Discord.
  Map<String, dynamic> buildOutgoing() {
    var map = new Map<String, dynamic>();
    map['op'] = opCode.code;
    map['d'] = innerPayload;
    return map;
  }
}
